import { LinkSpec } from "./LinkSpec";
/**
 *  Examines the character span matched by the pattern and determines
 *  if the match should be turned into an actionable link.
 *
 *  MatchFilter enables client code to have more control over
 *  what is allowed to match and become a link, and what is not.
 *
 *  For example:  when matching web URLs you would like things like
 *  http://www.example.com to match, as well as just example.com itelf.
 *  However, you would not want to match against the domain in
 *  support@example.com.  So, when matching against a web URL pattern you
 *  might also include a MatchFilter that disallows the match if it is
 *  immediately preceded by an at-sign (@).
 *
 *  @param s        The body of text against which the pattern
 *                  was matched
 *  @param start    The index of the first character in s that was
 *                  matched by the pattern - inclusive
 *  @param end      The index of the last character in s that was
 *                  matched - exclusive
 *
 *  @return         Whether this match should be turned into a link
 */
export declare type MatchFilter = (s: string, start: number, end: number) => boolean;
/**
 *  Examines the matched text and either passes it through or uses the
 *  data in the Matcher state to produce a replacement.
 *
 *  TransformFilter enables client code to have more control over
 *  how matched patterns are represented as URLs.
 *
 *  For example:  when converting a phone number such as (919)  555-1212
 *  into a tel: URL the parentheses, white space, and hyphen need to be
 *  removed to produce tel:9195551212.
 *
 *  @param match    The regex matcher state that found this URL text
 *  @param url      The text that was matched
 *
 *  @return         The transformed form of the URL
 */
export declare type TransformFilter = (match: RegExpExecArray, url: string) => string;
/**
 *  Bit field indicating that web URLs should be matched in methods that
 *  take an options mask
 */
export declare const WEB_URLS = 1;
/**
 *  Bit field indicating that email addresses should be matched in methods
 *  that take an options mask
 */
export declare const EMAIL_ADDRESSES = 2;
/**
 *  Bit mask indicating that all available patterns should be matched in
 *  methods that take an options mask
 *  <p><strong>Note:</strong></p> {@link #MAP_ADDRESSES} is deprecated.
 *  Use {@link android.view.textclassifier.TextClassifier#generateLinks(TextLinks.Request)}
 *  instead and avoid it even when targeting API levels where no alternative is available.
 */
export declare const ALL: number;
/**
 * Don't treat anything with fewer than this many digits as a
 * phone number.
 */
export declare const PHONE_NUMBER_MINIMUM_DIGITS = 5;
/**
 *  Scans the text of the provided Spannable and turns all occurrences of the link types
 *  indicated in the mask into clickable links. If the mask is nonzero, it also removes any
 *  existing URLSpans attached to the Spannable, to avoid problems if you call it repeatedly
 *  on the same text.
 *
 * @param text Spannable whose text is to be marked-up with links
 * @param mask mask to define which kinds of links will be searched
 * @param context Context to be used while identifying phone numbers
 * @param urlSpanFactory function used to create {@link URLSpan}s
 * @return true if at least one link is found and applied.
 */
export declare function addAutoLinks(text: string, mask?: number): LinkSpec[] | false;
/**
 * Applies a regex to a Spannable turning the matches into links.
 *
 * @param spannable       spannable whose text is to be marked-up with links.
 * @param pattern         regex pattern to be used for finding links.
 * @param defaultScheme   the default scheme to be prepended to links if the link does not
 *                        start with one of the <code>schemes</code> given.
 * @param schemes         array of schemes (eg <code>http://</code>) to check if the link found
 *                        contains a scheme. Passing a null or empty value means prepend
 *                        defaultScheme
 *                        to all links.
 * @param matchFilter     the filter that is used to allow the client code additional control
 *                        over which pattern matches are to be converted into links.
 * @param transformFilter filter to allow the client code to update the link found.
 * @param urlSpanFactory  function used to create {@link URLSpan}s
 *
 * @return True if at least one link is found and applied.
 */
export declare function addLinks(spannable: string, pattern: RegExp, defaultScheme: string | undefined, schemes: string[] | undefined, matchFilter: MatchFilter | undefined, transformFilter: TransformFilter): false | LinkSpec[];
