"use strict";
/*
 * Copyright (C) 2007 The Android Open Source Project
 * Copyright (C) 2019 Julien Roncaglia <julien@roncaglia.fr>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var patterns_1 = require("./patterns");
var log_1 = require("./log");
var LinkSpec_1 = require("./LinkSpec");
/**
 *  Bit field indicating that web URLs should be matched in methods that
 *  take an options mask
 */
exports.WEB_URLS = 0x01;
/**
 *  Bit field indicating that email addresses should be matched in methods
 *  that take an options mask
 */
exports.EMAIL_ADDRESSES = 0x02;
/**
 *  Bit mask indicating that all available patterns should be matched in
 *  methods that take an options mask
 *  <p><strong>Note:</strong></p> {@link #MAP_ADDRESSES} is deprecated.
 *  Use {@link android.view.textclassifier.TextClassifier#generateLinks(TextLinks.Request)}
 *  instead and avoid it even when targeting API levels where no alternative is available.
 */
exports.ALL = exports.WEB_URLS | exports.EMAIL_ADDRESSES;
/**
 * Don't treat anything with fewer than this many digits as a
 * phone number.
 */
exports.PHONE_NUMBER_MINIMUM_DIGITS = 5;
/**
 *  Filters out web URL matches that occur after an at-sign (@).  This is
 *  to prevent turning the domain name in an email address into a web link.
 */
var sUrlMatchFilter = function (s, start, end) {
    if (start == 0) {
        return true;
    }
    if (s.charAt(start - 1) == '@') {
        return false;
    }
    return true;
};
/**
 *  Filters out URL matches that don't have enough digits to be a
 *  phone number.
 */
var sPhoneNumberMatchFilter = function (s, start, end) {
    var digitCount = 0;
    for (var i = start; i < end; i++) {
        if (utils_1.isDigit(s.charAt(i))) {
            digitCount++;
            if (digitCount >= exports.PHONE_NUMBER_MINIMUM_DIGITS) {
                return true;
            }
        }
    }
    return false;
};
/**
 *  Transforms matched phone number text into something suitable
 *  to be used in a tel: URL.  It does this by removing everything
 *  but the digits and plus signs.  For instance:
 *  &apos;+1 (919) 555-1212&apos;
 *  becomes &apos;+19195551212&apos;
 */
var sPhoneNumberTransformFilter = function (match) { return patterns_1.digitsAndPlusOnly(match); };
/**
 *  Scans the text of the provided Spannable and turns all occurrences of the link types
 *  indicated in the mask into clickable links. If the mask is nonzero, it also removes any
 *  existing URLSpans attached to the Spannable, to avoid problems if you call it repeatedly
 *  on the same text.
 *
 * @param text Spannable whose text is to be marked-up with links
 * @param mask mask to define which kinds of links will be searched
 * @param context Context to be used while identifying phone numbers
 * @param urlSpanFactory function used to create {@link URLSpan}s
 * @return true if at least one link is found and applied.
 */
function addAutoLinks(text, mask) {
    if (mask === undefined) {
        mask = exports.ALL;
    }
    if (text != null && containsUnsupportedCharacters(text.toString())) {
        return false;
    }
    if (mask == 0) {
        return false;
    }
    var links = [];
    if ((mask & exports.WEB_URLS) != 0) {
        gatherLinks(links, text, patterns_1.AUTOLINK_WEB_URL, ["http://", "https://", "rtsp://"], sUrlMatchFilter, undefined);
    }
    if ((mask & exports.EMAIL_ADDRESSES) != 0) {
        gatherLinks(links, text, patterns_1.AUTOLINK_EMAIL_ADDRESS, ["mailto:"], undefined, undefined);
    }
    LinkSpec_1.pruneOverlaps(links);
    if (links.length == 0) {
        return false;
    }
    return links;
}
exports.addAutoLinks = addAutoLinks;
/**
 * Returns true if the specified text contains at least one unsupported character for applying
 * links. Also logs the error.
 *
 * @param text the text to apply links to
 * @hide
 */
function containsUnsupportedCharacters(text) {
    if (text.indexOf("\u202C") !== -1) {
        log_1.logError("Unsupported character for applying links: u202C");
        return true;
    }
    if (text.indexOf("\u202D") !== -1) {
        log_1.logError("Unsupported character for applying links: u202D");
        return true;
    }
    if (text.indexOf("\u202E") !== -1) {
        log_1.logError("Unsupported character for applying links: u202E");
        return true;
    }
    return false;
}
/**
 * Applies a regex to a Spannable turning the matches into links.
 *
 * @param spannable       spannable whose text is to be marked-up with links.
 * @param pattern         regex pattern to be used for finding links.
 * @param defaultScheme   the default scheme to be prepended to links if the link does not
 *                        start with one of the <code>schemes</code> given.
 * @param schemes         array of schemes (eg <code>http://</code>) to check if the link found
 *                        contains a scheme. Passing a null or empty value means prepend
 *                        defaultScheme
 *                        to all links.
 * @param matchFilter     the filter that is used to allow the client code additional control
 *                        over which pattern matches are to be converted into links.
 * @param transformFilter filter to allow the client code to update the link found.
 * @param urlSpanFactory  function used to create {@link URLSpan}s
 *
 * @return True if at least one link is found and applied.
 */
function addLinks(spannable, pattern, defaultScheme, schemes, matchFilter, transformFilter) {
    if (spannable != null && containsUnsupportedCharacters(spannable.toString())) {
        return false;
    }
    var schemesCopy;
    if (defaultScheme == null)
        defaultScheme = "";
    if (schemes == null || schemes.length < 1) {
        schemes = [];
    }
    schemesCopy = [];
    schemesCopy.push(defaultScheme.toLowerCase());
    for (var _i = 0, schemes_1 = schemes; _i < schemes_1.length; _i++) {
        var scheme = schemes_1[_i];
        schemesCopy.push((scheme == undefined) ? "" : scheme.toLowerCase());
    }
    var links = [];
    var m;
    while ((m = pattern.exec(spannable)) != null) {
        var start = m.index;
        var end = m.index + m[0].length;
        var allowed = true;
        if (matchFilter != null) {
            allowed = matchFilter(spannable, start, end);
        }
        if (allowed) {
            var url = makeUrl(m[0], schemesCopy, m, transformFilter);
            links.push({ url: url, start: start, end: end });
        }
    }
    return links;
}
exports.addLinks = addLinks;
function makeUrl(url, prefixes, matcher, filter) {
    if (filter !== undefined) {
        url = filter(matcher, url);
    }
    var hasPrefix = false;
    for (var i = 0; i < prefixes.length; i++) {
        if (url.length < prefixes[i].length) {
            continue;
        }
        var urlPrefixRange = url.substring(0, prefixes[i].length);
        if (urlPrefixRange.localeCompare(prefixes[i], undefined, { sensitivity: 'accent' }) === 0) {
            hasPrefix = true;
            // Fix capitalization if necessary
            if (urlPrefixRange.localeCompare(prefixes[i], undefined, { sensitivity: 'case' }) !== 0) {
                url = prefixes[i] + url.substring(prefixes[i].length);
            }
            break;
        }
    }
    if (!hasPrefix && prefixes.length > 0) {
        url = prefixes[0] + url;
    }
    return url;
}
function gatherLinks(links, s, pattern, schemes, matchFilter, transformFilter) {
    var m;
    while ((m = pattern.exec(s)) != null) {
        var start = m.index;
        var end = start + m[0].length;
        if (matchFilter === undefined || matchFilter(s, start, end)) {
            var url = makeUrl(m[0], schemes, m, transformFilter);
            var spec = { url: url, start: start, end: end };
            links.push(spec);
        }
    }
}
